﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_assassination.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for assassination rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local IsStealthed = clcInfo_Rogue.IsStealthed
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local CalculateTargetHealthPct = clcInfo_Rogue.CalculateTargetHealthPct
local UnitPower = UnitPower
local UnitAura = UnitAura
local UnitInRaid = UnitInRaid
local UnitInParty = UnitInParty
local IsSpellKnown = IsSpellKnown

-- Assign module variables
local specname = "assassination"
local prio_arg = "assassinationprio"
local sample_prio = "prep vanish ambush expose sb snd rupture vendetta"..
                    "envenom dispatch tricks mutilate"
local priority = {"none","none","ambush","none","none","snd","rupture",
                  "none","envenom","dispatch","none","mutilate"}
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    priority = priority,
    rangePerSkill = false,
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local Blindside = spells["Blindside"].name
local Envenom = spells["Envenom"].name
local FuryoftheDestroyer = spells["Fury of the Destroyer"].name
local Overkill = spells["Overkill"].name
local Rupture = spells["Rupture"].name
local SliceAndDice = spells["Slice and Dice"].name
local Stealth = spells["Stealth"].name
local Vanish = spells["Vanish"].name
local WeakenedArmor = spells["Weakened Armor"].name

local AmbushId = spells["Ambush"].id
local DispatchId = spells["Dispatch"].id
local EnvenomId = spells["Envenom"].id
local ExposeId = spells["Expose Armor"].id
local GarroteId = spells["Garrote"].id
local MutilateId = spells["Mutilate"].id
local PreparationId = spells["Preparation"].id
local RuptureId = spells["Rupture"].id
local ShadowBladesId = spells["Shadow Blades"].id
local ShadowFocusId = spells["Shadow Focus"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id
local VanishId = spells["Vanish"].id
local VendettaId = spells["Vendetta"].id

local actions  = {
    prep = {
        id = PreparationId,
        GetCD = function(cur_id)
            if IsSpellKnown(PreparationId) then
                if cur_id ~= PreparationId 
                   and (GetCooldown(VanishId) > 60 or cur_id == VanishId)
                then
                    return GetCooldown(PreparationId)
                end
                return 100
            end
            return 1000
        end,
        info = spells["Preparation"].name,
    },
    vanish = {
        id = VanishId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            if (not IsStealthed() or cur_id == AmbushId)
               and ((anticipation_stacks 
                     and anticipation_stacks<4)
                    or combo_points < 4)
            then
                --pool energy if no shadow focus
                local yellow = ((UnitPower("player") < 61)
                                and not (IsSpellKnown(ShadowFocusId)))
                local vanishcd = ((cur_id == PreparationId) and 0)
                                  or GetCooldown(VanishId)
                return max(vanishcd,
                           (GetTimeLeft("player", ShadowBlades, "HELPFUL|PLAYER"))),
                       yellow
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Vanish"].name,
    },
    ambush = {
        id = AmbushId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= AmbushId and IsStealthed() then
                return GetCooldown(AmbushId)
            end
            return 100
        end,
        build = 2,
        info = spells["Ambush"].name,
    },
    expose = {
        id = ExposeId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            if combo_points < 5
               or (anticipation_stacks and anticipation_stacks < 5)
            then
                local time_left, count 
                             = GetTimeLeft("target", WeakenedArmor, "HARMFUL")
                if not count then count = 0 end
                if cur_id == ExposeId then count = count + 1 end
                if count < 3 or (time_left < 3 and cur_id ~= ExposeId) then
                    return GetCooldown(ExposeId)
                end
            end
            return 100
        end,
        build = 1,
        info = spells["Expose Armor"].name,
    },
    sb = {
        id = ShadowBladesId,
        GetCD = function(cur_id)
            if cur_id ~= ShadowBladesId then
                return GetCooldown(ShadowBladesId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Shadow Blades"].name,
    },
    snd = {
        id = SliceAndDiceId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= SliceAndDiceId 
               and combo_points > 0
            then
                return max(GetCooldown(SliceAndDiceId),
                           (GetTimeLeft("player", SliceAndDice, "HELPFUL|PLAYER"))-2)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Slice and Dice"].name,
    },
    rupture = {
        id = RuptureId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId then
                if combo_points == 5 then
                   return max(GetCooldown(RuptureId),
                          (GetTimeLeft("target", Rupture, "HARMFUL|PLAYER"))-6)
                elseif combo_points > 0 then
                   return max(GetCooldown(RuptureId),
                          (GetTimeLeft("target", Rupture, "HARMFUL|PLAYER"))-4)
                end
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Rupture"].name,
    },
    vendetta = {
        id = VendettaId,
        GetCD = function(cur_id)
            if cur_id ~= VendettaId then
                return GetCooldown(VendettaId)
            end
            return 100
        end,
        info = spells["Vendetta"].name,
    },
    envenom = {
        id = EnvenomId,
        GetCD = function(_, combo_points)
            local envenom_cd = GetCooldown(EnvenomId)
            if combo_points > 1
               and GetTimeLeft("player", SliceAndDice, "HELPFUL") < 3 then
                return envenom_cd
            end
            if combo_points > 4 then
                local envenom_left = GetTimeLeft("player", Envenom, "PLAYER")
                local yellow = false
                if UnitPower("player") > 90
                   or GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0
                then
                    return envenom_cd
                elseif envenom_left > envenom_cd then
                    envenom_cd = envenom_left
                    yellow = true -- yellow icon so user knows to wait
                end
                return envenom_cd, yellow
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Envenom"].name,
    },
    dispatch = {
        id = DispatchId,
        GetCD = function(cur_id, combo_points)
            if combo_points < 5 
               and (CalculateTargetHealthPct() < 35
                    or ((GetTimeLeft("player", Blindside, "HELPFUL"))>0
                        and cur_id ~= DispatchId))then
                return GetCooldown(DispatchId)
            end
            return 100
        end,
        build = 1,
        info = spells["Dispatch"].name,
    },
    tricks = {
        id = TricksId,
        GetCD = function(cur_id)
            if cur_id ~= TricksId and (UnitInRaid("player")
                                       or UnitInParty("player")) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Tricks of the Trade"].name,
    },
    mutilate = {
        id = MutilateId,
        GetCD = function(_, combo_points)
            if combo_points < 5 and CalculateTargetHealthPct() >= 35 then
                return GetCooldown(MutilateId)
            end
            return 100
        end,
        build = 2,
        info = spells["Mutilate"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
        = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions,
                                          tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadAssassinationModule()
    local _, specialization_name = GetSpecializationInfo(1)
    clcInfo_Rogue.LoadModule(class_module_name, specialization_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1]
                                                    = LoadAssassinationModule

-- giving queue as command line
local function CmdAssassination(args)
    clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdAssassination

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconAssassinationCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconAssassinationNext(...)
    return IconNext()
end